#ifndef ABSTRACTOBJ_H
#define ABSTRACTOBJ_H

#include "abstractproperty.h"
#include <QList>
#include <QString>
#include <QDebug>

class AbstractObj
{
public:
    enum ObjType {Root,
                  Version,
                  Include,
                  Define,
                  Position,
                  List,
                  Dialog,
                  CheckBox,
                  Radio,
                  Spinner,
                  Edit,
                  VarList,
                  VarName,
                  File,
                  ListBox,
                  ComboBox,
                  Button,
                  Text,
                  TextBox,
                  GroupBox,
                  Frame,
                  Color,
                  Exp,
                  Ok,
                  Submit,
                  Cancel,
                  Copy,
                  Help,
                  Reset,
                  Program,
                  Script};
    AbstractObj(AbstractObj::ObjType objType);

    // Pure virtual functions
    virtual QString commandString() const = 0;
    virtual QString commandNameForDisplayRoleData() const = 0;
    virtual QString typeNameForDomElement() const = 0;
    virtual bool hasName() const = 0;

    // Not pure virtual functions
    virtual QDomElement domElement(QDomDocument &xmlDocument) const;
    virtual void setFromObjDomElement(const QDomElement &domElement);

    // Tree management functions
    int row() const;
    AbstractObj *objAt(int row) const;
    AbstractObj *parentObj();
    inline int objCount() const {return m_childObjs.count();}
    void appendObj(AbstractObj *obj);
    void insertObj(AbstractObj *obj, int pos);
    void swapObjects(int pos1, int pos2);
    void deleteObjAt(int pos);
    void deleteChildren();

    // Other functions
    void appendProperty(AbstractProperty *property);
    QVariant displayRoleDataForPropertyAt(int pos) const;
    QVariant displayRoleDataAt(int pos) const;
    QVariant editRoleDataAt(int pos) const;
    bool setEditRoleDataForPropertyAt(int pos, QVariant data);
    inline AbstractObj::ObjType objType() {return m_objType;}
    int propertiesCount() const;
    AbstractProperty *propertyAt(int pos);
    bool isControlObj() const;
    bool hasAtLeastOneChildControlObjBeforeRow(int row) const;

protected:
    QString standardInnerCommandString() const;

private:
    AbstractObj::ObjType m_objType;
    QList<AbstractProperty*> m_properties;
    AbstractObj* m_parentObj;
    QList<AbstractObj*> m_childObjs;
};

Q_DECLARE_METATYPE(AbstractObj::ObjType)

#endif // ABSTRACTOBJ_H
