#ifndef CANVAS_H
#define CANVAS_H

#include <QAbstractItemView>
#include <QScrollBar>
#include "geometrydataproperty.h"
#include "abstractobjforcanvas.h"
#include "abstractcontrolobj.h"
#include <QFont>

typedef struct ObjsForCanvasInDialog_ {QHash<int, AbstractObjForCanvas*> objs;} ObjsForCanvasInDialog;

class Canvas : public QAbstractItemView
{
Q_OBJECT

public:
    explicit Canvas(QWidget *parent);

    // Re-implemented pure virtual functions
    virtual QRect visualRect(const QModelIndex &index) const;
    virtual bool isIndexHidden(const QModelIndex &index) const;
    virtual void scrollTo(const QModelIndex &index, ScrollHint hint);
    virtual QModelIndex indexAt(const QPoint &point) const;

    // Re-implemented not-pure virtual functions
    virtual void setModel(QAbstractItemModel *model);

    // Other functions
    void resetRowOfSelectedDialog();

private:    
    // Data
    mutable int idealWidth;
    mutable int idealHeight;
    mutable QHash<int, ObjsForCanvasInDialog> objsForCanvas;
    mutable bool hashIsDirty;
    int rowOfSelectedDialog;
    QFont stataDialogFont;

    // Functions
    void calculateRectsIfNecessary() const;
    QRect viewportRectForControlObj(int dialogRow, int controlRow) const;
    QRect rectForControlObj(AbstractControlObj *controlObj, QRect previousRect) const;
    int nextGeometryValue(int previousGeometryValue, AbstractControlObj *controlObj,
                          GeometryDataProperty::GeometryDataType geometryDataType) const;
    int defaultDimForObj(AbstractControlObj *controlObj,
                         GeometryDataProperty::GeometryDataType geometryDataType) const;
    bool isControlObjType(AbstractObj::ObjType objType);
    void paintOutline(QPainter *painter, const QRect &rectangle);

protected:
    // Re-implemented pure virtual functions
    virtual QRegion visualRegionForSelection(const QItemSelection &selection) const;
    virtual QModelIndex moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers);
    virtual int horizontalOffset() const;
    virtual int verticalOffset() const;
    virtual void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command);

    // Re-implemented not-pure virtual functions
    virtual void scrollContentsBy(int dx, int dy);
    virtual void resizeEvent(QResizeEvent *event);
    virtual void dropEvent(QDropEvent *event);

    // Other functions
    virtual void paintEvent(QPaintEvent *event);

public slots:
    void onSelectionChangedInObjectListView(const QItemSelection &selected,
                                            const QItemSelection &deselected);

protected slots:
    // Re-implemented not-pure virtual functions
    virtual void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    virtual void rowsInserted(const QModelIndex &parent, int start, int end);
    virtual void rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end);
    virtual void updateGeometries();
    virtual void mousePressEvent(QMouseEvent *event);
};

#endif // CANVAS_H
