#include "colorsetter.h"
#include <QHBoxLayout>
#include <QLabel>
#include <QMenu>
#include <QColorDialog>

ColorSetter::ColorSetter(QWidget *parent)
    : AbstractSetter(parent)
{
    // Horizontal layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Spinboxes
    // Red
    redLabel = new QLabel("R:");
    layout->addWidget(redLabel);
    redSpinBox = new QSpinBox(this);
    redSpinBox->setMinimum(0);
    redSpinBox->setMaximum(255);
    redSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    layout->addWidget(redSpinBox);
    // Green
    greenLabel = new QLabel("G:");
    layout->addWidget(greenLabel);
    greenSpinBox = new QSpinBox(this);
    greenSpinBox->setMinimum(0);
    greenSpinBox->setMaximum(255);
    greenSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    layout->addWidget(greenSpinBox);
    // Blue
    blueLabel = new QLabel("B:");
    layout->addWidget(blueLabel);
    blueSpinBox = new QSpinBox(this);
    blueSpinBox->setMinimum(0);
    blueSpinBox->setMaximum(255);
    blueSpinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    layout->addWidget(blueSpinBox);

    // Not specified label
    notSpecifiedLabel = new QLabel("Not specified");
    notSpecifiedLabel->hide();
    layout->addWidget(notSpecifiedLabel);

    // Main menu
    QMenu *menu = new QMenu(this);
    advancedColorPeekerAction = new QAction("Advanced color peeker", this);
    connect(advancedColorPeekerAction, SIGNAL(triggered()),
            this, SLOT(onAdvancedColorPeekerActionTriggered()));
    menu->addAction(advancedColorPeekerAction);
    simpleColorPeekerAction = new QAction("Simple color peeker", this);
    connect(simpleColorPeekerAction, SIGNAL(triggered()), this, SLOT(onSimpleColorPeekerActionTriggered()));
    simpleColorPeekerAction->setDisabled(true);
    menu->addAction(simpleColorPeekerAction);
    menu->addSeparator();
    setNotSpecifiedAction = new QAction("Not specified", this);
    connect(setNotSpecifiedAction, SIGNAL(triggered()), this, SLOT(onSetNotSpecifiedActionTriggered()));
    menu->addAction(setNotSpecifiedAction);

    // Button
    button = new QPushButton;
    button->setMenu(menu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);
}

void ColorSetter::setData(const QString &rgbString)
{
    if (rgbString.isEmpty())
    {
        redLabel->hide();
        redSpinBox->hide();
        greenLabel->hide();
        greenSpinBox->hide();
        blueLabel->hide();
        blueSpinBox->hide();
        notSpecifiedLabel->show();
        m_isSpecified = false;
        simpleColorPeekerAction->setEnabled(true);
    }

    QStringList rgbStringList = rgbString.split(" ");
    if (rgbStringList.isEmpty()) return;
    if (rgbStringList.count() != 3) return;

    bool okConversionToInt;
    int red = rgbStringList.at(0).toInt(&okConversionToInt);
    if (okConversionToInt)
        redSpinBox->setValue(red);
    int green = rgbStringList.at(1).toInt(&okConversionToInt);
    if (okConversionToInt)
        greenSpinBox->setValue(green);
    int blue = rgbStringList.at(2).toInt(&okConversionToInt);
    if (okConversionToInt)
        blueSpinBox->setValue(blue);

    redLabel->show();
    redSpinBox->show();
    greenLabel->show();
    greenSpinBox->show();
    blueLabel->show();
    blueSpinBox->show();
    notSpecifiedLabel->hide();
    m_isSpecified = true;
    simpleColorPeekerAction->setEnabled(false);
}

QString ColorSetter::data()
{
    if (m_isSpecified)
        return QString("%1 %2 %3").arg(redSpinBox->value())
                                  .arg(greenSpinBox->value())
                                   .arg(blueSpinBox->value());
    else
        return "";
}

void ColorSetter::onAdvancedColorPeekerActionTriggered()
{
    QColor initialColor;
    if (m_isSpecified)
    {
        initialColor.setRed(redSpinBox->value());
        initialColor.setGreen(greenSpinBox->value());
        initialColor.setBlue(blueSpinBox->value());
    }
    else
    {
        initialColor.setRed(255);
        initialColor.setGreen(0);
        initialColor.setBlue(0);
    }

    QColor color = QColorDialog::getColor(initialColor);
    if (color.isValid())
    {
        redSpinBox->setValue(color.red());
        greenSpinBox->setValue(color.green());
        blueSpinBox->setValue(color.blue());
        redLabel->show();
        redSpinBox->show();
        greenLabel->show();
        greenSpinBox->show();
        blueLabel->show();
        blueSpinBox->show();
        notSpecifiedLabel->hide();
        simpleColorPeekerAction->setEnabled(false);
        m_isSpecified = true;
    }
}

void ColorSetter::onSimpleColorPeekerActionTriggered()
{
    redLabel->show();
    redSpinBox->show();
    greenLabel->show();
    greenSpinBox->show();
    blueLabel->show();
    blueSpinBox->show();
    notSpecifiedLabel->hide();
    simpleColorPeekerAction->setEnabled(false);
    m_isSpecified = true;
}

void ColorSetter::onSetNotSpecifiedActionTriggered()
{
    redLabel->hide();
    redSpinBox->hide();
    greenLabel->hide();
    greenSpinBox->hide();
    blueLabel->hide();
    blueSpinBox->hide();
    notSpecifiedLabel->show();
    m_isSpecified = false;
    simpleColorPeekerAction->setEnabled(true);
}

