#include "debugreportwindow.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QToolButton>
#include <QPushButton>

#include <QDebug>

DebugReportWindow::DebugReportWindow(Document *document, QWidget *parent) :
    QWidget(parent)
{
    setAttribute(Qt::WA_GroupLeader);
    setWindowFlags(Qt::Window);
    setWindowTitle("Visua - Debug report");

    // Debug report model
    debugReportModel = new DebugReportModel(document, this);

    // Main layout
    QVBoxLayout *mainLayout = new QVBoxLayout;
    setLayout(mainLayout);

    // Splitter
    splitter = new QSplitter(Qt::Vertical);
    mainLayout->addWidget(splitter);

    // Issues report
    QWidget *debugIssuesGroup = new QWidget;
    splitter->addWidget(debugIssuesGroup);
    QLabel *debugIssuesLabel = new QLabel("Issues:");
    debugReportView = new DebugReportView(this);
    debugReportView->setModel(debugReportModel);
    QVBoxLayout *debugIssuesLayout = new QVBoxLayout;
    debugIssuesLayout->setMargin(0);
    debugIssuesGroup->setLayout(debugIssuesLayout);
    debugIssuesLayout->addWidget(debugIssuesLabel);
    debugIssuesLayout->addWidget(debugReportView);
    splitter->addWidget(debugIssuesGroup);

    // More on issues
    QWidget *moreOnIssuesGroup = new QWidget;
    splitter->addWidget(moreOnIssuesGroup);
    splitter->setChildrenCollapsible(false);
    QLabel *moreOnIssuesLabel = new QLabel("More on issue:");
    textEdit = new QTextEdit;
    textEdit->setReadOnly(true);
    QVBoxLayout *moreOnIssuesLayout = new QVBoxLayout;
    moreOnIssuesLayout->setMargin(0);
    moreOnIssuesGroup->setLayout(moreOnIssuesLayout);
    moreOnIssuesLayout->addWidget(moreOnIssuesLabel);
    moreOnIssuesLayout->addWidget(textEdit);
    splitter->addWidget(moreOnIssuesGroup);

    // Buttons
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    mainLayout->addLayout(buttonsLayout);
    QToolButton *debugToolButton = new QToolButton;
    debugToolButton->setIcon(QIcon(":/images/icons/bug.png"));
    debugToolButton->setToolTip("Update debug report");
    buttonsLayout->addWidget(debugToolButton);
    buttonsLayout->addStretch();
    QPushButton *closeButton = new QPushButton("Close");
    buttonsLayout->addWidget(closeButton);

    // Connections
    connect(debugToolButton, SIGNAL(clicked()), this, SLOT(updateReport()));
    connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
    connect(debugReportView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChangedInDebugReportView(QItemSelection,QItemSelection)));
}

void DebugReportWindow::updateReport()
{
    debugReportModel->updateReport();
}

void DebugReportWindow::onSelectionChangedInDebugReportView(const QItemSelection &selected,
                                                            const QItemSelection &deselected)
{
    Q_UNUSED(deselected)

    if (selected.isEmpty())
        textEdit->setText("");
    else
    {
        QString moreOnIssue = selected.at(0).indexes().at(0).data(Qt::EditRole).toString();
        textEdit->setText(moreOnIssue);
    }
}

QByteArray DebugReportWindow::splitterSaveState()
{
    return splitter->saveState();
}

void DebugReportWindow::restoreSplitterState(const QByteArray &state)
{
    splitter->restoreState(state);
}
