#include "defaultsetter.h"
#include <QHBoxLayout>
#include <QDebug>

DefaultSetter::DefaultSetter(DefaultSetterType defaultSetterType, QWidget *parent)
    : AbstractSetter(parent)
{
    m_defaultSetterType = defaultSetterType;
    createMenu();

    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Line edit
    lineEdit = new QLineEdit;
    lineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    lineEdit->setFocus();
    lineEdit->setCursorPosition(1);
    layout->addWidget(lineEdit);

    // Spin box
    spinBox = new QSpinBox;
    spinBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    spinBox->hide();
    layout->addWidget(spinBox);

    // Button
    button = new QPushButton;
    button->setMenu(mainMenu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);
}

void DefaultSetter::setDefaultValue(const QString &value)
{
    bool conversionIsOk;
    int num = value.toInt(&conversionIsOk);
    if (conversionIsOk)
    {
        spinBox->show();
        lineEdit->hide();
        spinBox->setValue(num);
    }
    else
    {
        spinBox->hide();
        lineEdit->show();
        lineEdit->setText(value);
    }
}

QString DefaultSetter::defaultValue()
{
    if (lineEdit->isVisible())
        return lineEdit->text();
    else
        return QString("%1").arg(spinBox->value());
}

void DefaultSetter::createMenu()
{
    mainMenu = new QMenu(this);

    // numberSpecifiedAction
    if (m_defaultSetterType == DefaultSetter::NumVal)
    {
        QAction *numberSpecifiedAction = new QAction("Specified number", this);
        mainMenu->addAction(numberSpecifiedAction);
        connect(numberSpecifiedAction, SIGNAL(triggered()), this, SLOT(onNumberSpecifiedActionTriggered()));
    }

    // cNameAction
    QAction *cNameAction;
    switch (m_defaultSetterType)
    {
        case DefaultSetter::NumVal:
            cNameAction = new QAction("Value of c(name)", this);
            break;
        case DefaultSetter::StrVal:
            cNameAction = new QAction("Contents of c(name)", this);
            break;
        default:
            cNameAction = 0;
            break;
    }
    mainMenu->addAction(cNameAction);
    connect(cNameAction, SIGNAL(triggered()), this, SLOT(onCNameActionTriggered()));

    // rNameAction
    QAction *rNameAction;
    switch (m_defaultSetterType)
    {
        case DefaultSetter::NumVal:
            rNameAction = new QAction("Value of r(name)", this);
            break;
        case DefaultSetter::StrVal:
            rNameAction = new QAction("Contents of r(name)", this);
            break;
        default:
            rNameAction = 0;
            break;
    }
    mainMenu->addAction(rNameAction);
    connect(rNameAction, SIGNAL(triggered()), this, SLOT(onRNameActionTriggered()));

    // eNameAction
    QAction *eNameAction;
    switch (m_defaultSetterType)
    {
        case DefaultSetter::NumVal:
            eNameAction = new QAction("Value of e(name)", this);
            break;
        case DefaultSetter::StrVal:
            eNameAction = new QAction("Contents of e(name)", this);
            break;
        default:
            eNameAction = 0;
            break;
    }
    mainMenu->addAction(eNameAction);
    connect(eNameAction, SIGNAL(triggered()), this, SLOT(onENameActionTriggered()));

    // sNameAction
    QAction *sNameAction;
    switch (m_defaultSetterType)
    {
        case DefaultSetter::NumVal:
            sNameAction = new QAction("Value of s(name)", this);
            break;
        case DefaultSetter::StrVal:
            sNameAction = new QAction("Contents of s(name)", this);
            break;
        default:
            sNameAction = 0;
            break;
    }
    mainMenu->addAction(sNameAction);
    connect(sNameAction, SIGNAL(triggered()), this, SLOT(onSNameActionTriggered()));

    if (m_defaultSetterType == DefaultSetter::StrVal)
    {
        QAction *charAction = new QAction("Value of characteristic", this);
        mainMenu->addAction(charAction);
        connect(charAction, SIGNAL(triggered()), this, SLOT(onCharActionTriggered()));
    }

    // globalNameAction
    QAction *globalNameAction = new QAction("Global macro", this);
    mainMenu->addAction(globalNameAction);
    connect(globalNameAction, SIGNAL(triggered()), this, SLOT(onGlobalNameActionTriggered()));

    // Separator
    mainMenu->addSeparator();

    // clearAction
    QAction *clearAction = new QAction("Clear", this);
    mainMenu->addAction(clearAction);
    connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));
}

void DefaultSetter::onNumberSpecifiedActionTriggered()
{
    spinBox->show();
    lineEdit->hide();
}

void DefaultSetter::onCNameActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("c(name)");
    lineEdit->setSelection(2, 4);
    lineEdit->setFocus();
}

void DefaultSetter::onRNameActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("r(name)");
    lineEdit->setSelection(2, 4);
    lineEdit->setFocus();
}

void DefaultSetter::onENameActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("e(name)");
    lineEdit->setSelection(2, 4);
    lineEdit->setFocus();
}

void DefaultSetter::onSNameActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("s(name)");
    lineEdit->setSelection(2, 4);
    lineEdit->setFocus();
}

void DefaultSetter::onCharActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("char varname[charname]");
    lineEdit->setSelection(5, 17);
    lineEdit->setFocus();
}

void DefaultSetter::onGlobalNameActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("global name");
    lineEdit->setSelection(7, 4);
    lineEdit->setFocus();
}

void DefaultSetter::onClearActionTriggered()
{
    spinBox->hide();
    lineEdit->show();
    lineEdit->setText("");
    lineEdit->setFocus();
}
