#include "definedefinitionsetter.h"
#include <QHBoxLayout>
#include <QDebug>

DefineDefinitionSetter::DefineDefinitionSetter(QWidget *parent)
    : AbstractSetter(parent)
{
    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Spin box
    spinBox = new QSpinBox(this);
    spinBox->setMinimum(0);
    spinBox->setMaximum(9999);
    layout->addWidget(spinBox);

    // Combo box
    comboBox = new QComboBox(this);
    comboBox->addItem("Default"); // 0
    comboBox->addItem("Number specified"); // 1
    comboBox->addItem("Positive offset"); // 2
    comboBox->addItem("Negative offset"); // 3
    comboBox->addItem("Previous x"); // 4
    comboBox->addItem("Previous y"); // 5
    comboBox->addItem("Previous width"); // 6
    comboBox->addItem("Previous height"); // 7
    comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentComboBoxItemChanged(int)));
    layout->addWidget(comboBox);

}

void DefineDefinitionSetter::setData(const QString &data)
{
    bool isNumeric;
    int numericValue = data.toInt(&isNumeric);

    // Data should not be empty
    if (data.isEmpty())
    {
        spinBox->hide();
        comboBox->setCurrentIndex(0);
    }

    if (isNumeric && data.at(0) != '+' && data.at(0) != '-') // Number specified
    {
        spinBox->setPrefix("");
        spinBox->setValue(numericValue);
        spinBox->show();
        comboBox->setCurrentIndex(1);
    }
    else if (data == ".")
    {
        spinBox->hide();
        comboBox->setCurrentIndex(0);
    }
    else if (data.at(0) == '+') // Positive offset
    {
        spinBox->setPrefix("+");
        QString implicitValue = data.mid(1);
        spinBox->setValue(implicitValue.toInt());
        spinBox->show();
        comboBox->setCurrentIndex(2);
    }
    else if (data.at(0) == '-') // Negative offset
    {
        spinBox->setPrefix("-");
        QString implicitValue = data.mid(1);
        spinBox->setValue(implicitValue.toInt());
        spinBox->show();
        comboBox->setCurrentIndex(3);
    }
    else if (data == "@x")
    {
        spinBox->hide();
        comboBox->setCurrentIndex(4);
    }
    else if (data == "@y")
    {
        spinBox->hide();
        comboBox->setCurrentIndex(5);
    }
    else if (data == "@xsize")
    {
        spinBox->hide();
        comboBox->setCurrentIndex(6);
    }
    else if (data == "@ysize")
    {
        spinBox->hide();
        comboBox->setCurrentIndex(7);
    }
}

QString DefineDefinitionSetter::data()
{
    switch (comboBox->currentIndex())
    {
        case 0: // Default
            return ".";
            break;
        case 1: // Number specified
            return QString("%1").arg(spinBox->value());
            break;
        case 2: // Positive offset
            return QString("+%1").arg(spinBox->value());
            break;
        case 3: // Negative offset
            return QString("-%1").arg(spinBox->value());
            break;
        case 4: // Previous x
            return "@x";
            break;
        case 5: // Previous y
            return "@y";
            break;
        case 6: // Previuos width
            return "@xsize";
            break;
        case 7: // Previous height
            return "@ysize";
            break;
        default:
            return QString();
            break;
    }
}

void DefineDefinitionSetter::onCurrentComboBoxItemChanged(int pos)
{
    switch (pos)
    {
        case 0: // Default
            spinBox->hide();
            break;
        case 1: // Number specified
            spinBox->setPrefix("");
            spinBox->show();
            break;
        case 2: // Positive offset
            spinBox->setPrefix("+");;
            spinBox->show();
            break;
        case 3: // Negative offset
            spinBox->setPrefix("-");
            spinBox->show();
            break;
        case 4: // Previous x
            spinBox->hide();
            break;
        case 5: // Previous y
            spinBox->hide();
            break;
        case 6: // Previuos width
            spinBox->hide();
            break;
        case 7: // Previous height
            spinBox->hide();
            break;
        default:
            break;
    }
}
