#include "dialogobj.h"

// Properties
#include "genericplainstringproperty.h"
#include "genericlabeledstringproperty.h"

// Objects
#include "checkboxobj.h"
#include "radioobj.h"
#include "spinnerobj.h"
#include "editobj.h"
#include "varlistobj.h"
#include "varnameobj.h"
#include "fileobj.h"
#include "listboxobj.h"
#include "comboboxobj.h"
#include "buttonobj.h"
#include "textobj.h"
#include "textboxobj.h"
#include "groupboxobj.h"
#include "frameobj.h"
#include "colorobj.h"
#include "expobj.h"
#include "includeobj.h"

DialogObj::DialogObj()
    : AbstractObj(AbstractObj::Dialog)
{
    GenericPlainStringProperty *nameProperty
            = new GenericPlainStringProperty("name", "Name", AbstractDelegate::Name);
    nameProperty->setValue("mydialog");
    appendProperty(nameProperty);

    GenericLabeledStringProperty *dialogTitleProperty
            = new GenericLabeledStringProperty("title", "Title", AbstractDelegate::GenericString);
    appendProperty(dialogTitleProperty);

    GenericLabeledStringProperty *dialogTabTitleProperty
            = new GenericLabeledStringProperty("tabtitle", "Tab title", AbstractDelegate::GenericString);
    appendProperty(dialogTabTitleProperty);
}

QString DialogObj::commandString() const
{
    QString string;
    string += "DIALOG" + standardInnerCommandString();
    string += "\nBEGIN";
    for (int i=0; i<objCount(); i++)
        string += "\n  " + objAt(i)->commandString();
    string += "\nEND";
    return string;
}

QString DialogObj::commandNameForDisplayRoleData() const
{
    return "Dialog";
}

QString DialogObj::typeNameForDomElement() const
{
    return "dialog";
}

bool DialogObj::hasName() const
{
    return true;
}

void DialogObj::setFromObjDomElement(const QDomElement &domElement)
{
    deleteChildren();
    AbstractObj::setFromObjDomElement(domElement);

    QDomNode childrenDomNode = domElement.namedItem("children");
    if (!childrenDomNode.isNull())
    {
        QDomElement childrenDomElement = childrenDomNode.toElement();
        QDomNode childDomNode = childrenDomElement.firstChild();
        while (!childDomNode.isNull())
        {
            if (childDomNode.toElement().tagName() == "object")
            {
                QDomElement childDomElement = childDomNode.toElement();
                QString objectType = childDomElement.attributes().namedItem("type").nodeValue();
                if (objectType == "checkbox")
                {
                    CheckBoxObj *checkBox = new CheckBoxObj;
                    checkBox->setFromObjDomElement(childDomElement);
                    appendObj(checkBox);
                }
                if (objectType == "radio")
                {
                    RadioObj *radioObj = new RadioObj;
                    radioObj->setFromObjDomElement(childDomElement);
                    appendObj(radioObj);
                }
                if (objectType == "spinner")
                {
                    SpinnerObj *spinnerObj = new SpinnerObj;
                    spinnerObj->setFromObjDomElement(childDomElement);
                    appendObj(spinnerObj);
                }
                if (objectType == "edit")
                {
                    EditObj *editObj = new EditObj;
                    editObj->setFromObjDomElement(childDomElement);
                    appendObj(editObj);
                }
                if (objectType == "varlist")
                {
                    VarListObj *varListObj = new VarListObj;
                    varListObj->setFromObjDomElement(childDomElement);
                    appendObj(varListObj);
                }
                if (objectType == "varname")
                {
                    VarNameObj *varNameObj = new VarNameObj;
                    varNameObj->setFromObjDomElement(childDomElement);
                    appendObj(varNameObj);
                }
                if (objectType == "file")
                {
                    FileObj *fileObj = new FileObj;
                    fileObj->setFromObjDomElement(childDomElement);
                    appendObj(fileObj);
                }
                if (objectType == "listbox")
                {
                    ListBoxObj *listBoxObj = new ListBoxObj;
                    listBoxObj->setFromObjDomElement(childDomElement);
                    appendObj(listBoxObj);
                }
                if (objectType == "combobox")
                {
                    ComboBoxObj *comboBoxObj = new ComboBoxObj;
                    comboBoxObj->setFromObjDomElement(childDomElement);
                    appendObj(comboBoxObj);
                }
                if (objectType == "button")
                {
                    ButtonObj *buttonObj = new ButtonObj;
                    buttonObj->setFromObjDomElement(childDomElement);
                    appendObj(buttonObj);
                }
                if (objectType == "text")
                {
                    TextObj *textObj = new TextObj;
                    textObj->setFromObjDomElement(childDomElement);
                    appendObj(textObj);
                }
                if (objectType == "textbox")
                {
                    TextBoxObj *textBoxObj = new TextBoxObj;
                    textBoxObj->setFromObjDomElement(childDomElement);
                    appendObj(textBoxObj);
                }
                if (objectType == "groupbox")
                {
                    GroupBoxObj *groupBoxObj = new GroupBoxObj;
                    groupBoxObj->setFromObjDomElement(childDomElement);
                    appendObj(groupBoxObj);
                }
                if (objectType == "frame")
                {
                    FrameObj *frameObj = new FrameObj;
                    frameObj->setFromObjDomElement(childDomElement);
                    appendObj(frameObj);
                }
                if (objectType == "color")
                {
                    ColorObj *colorObj = new ColorObj;
                    colorObj->setFromObjDomElement(childDomElement);
                    appendObj(colorObj);
                }
                if (objectType == "exp")
                {
                    ExpObj *expObj = new ExpObj;
                    expObj->setFromObjDomElement(childDomElement);
                    appendObj(expObj);
                }
                if (objectType == "include")
                {
                    IncludeObj *includeObj = new IncludeObj;
                    includeObj->setFromObjDomElement(childDomElement);
                    appendObj(includeObj);
                }
            }
            childDomNode = childDomNode.nextSibling();
        }
    }
}

QDomElement DialogObj::domElement(QDomDocument &xmlDocument) const
{
    QDomElement element = AbstractObj::domElement(xmlDocument);

    QDomElement childrenElement = xmlDocument.createElement("children");
    if (objCount()>0)
    {
        for (int i=0; i<objCount(); i++)
            childrenElement.appendChild(objAt(i)->domElement(xmlDocument));
        element.appendChild(childrenElement);
    }

    return element;
}
