#include "geometrydatadelegate.h"
#include <QSortFilterProxyModel>
#include "geometrydatasetter.h"
#include "abstractobj.h"
#include "defineobj.h"
#include <QMessageBox>
#include <QDebug>

GeometryDataDelegate::GeometryDataDelegate(QObject *parent)
    : AbstractDelegate(parent)
{
}

QWidget *GeometryDataDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,
              const QModelIndex &index) const
{
    Q_UNUSED(option)
    Q_UNUSED(index)

    const QSortFilterProxyModel *proxyModel = static_cast<const QSortFilterProxyModel*>(index.model());
    QModelIndex sourceIndex = proxyModel->mapToSource(index);
    const QAbstractItemModel *sourceModel = sourceIndex.model();
    QStringList userDefinedNameList;
    for (int i=0; i<sourceModel->rowCount(); i++)
    {
        QModelIndex ind = sourceModel->index(i, 0, QModelIndex());
        AbstractObj *obj = static_cast<AbstractObj*>(ind.internalPointer());
        if (obj->objType() == AbstractObj::Define)
        {
            DefineObj *defineObj = static_cast<DefineObj*>(obj);
            userDefinedNameList << defineObj->nameProperty->editRoleData().toString();
        }
    }

    GeometryDataSetter *geometryDataSetter = new GeometryDataSetter(userDefinedNameList, parent);
    return geometryDataSetter;
}

void GeometryDataDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    GeometryDataSetter *geometryDataSetter = static_cast<GeometryDataSetter*>(editor);
    geometryDataSetter->setGeometryData(index.data(Qt::EditRole).toString());
}

void GeometryDataDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                            const QModelIndex &index) const
{
    GeometryDataSetter *geometryDataSetter = static_cast<GeometryDataSetter*>(editor);
    QString data = geometryDataSetter->geometryData();
    if (data.isEmpty())
        QMessageBox::warning(geometryDataSetter, "Visua", "Name can't be empty");
    else
        model->setData(index, data);
}

