#include "helpcontentsview.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QDir>
#include <QDebug>

HelpContentsView::HelpContentsView(const QString &page, QWidget *parent) :
    QWidget(parent)
{
    setAttribute(Qt::WA_GroupLeader);
    setWindowFlags(Qt::Window);

    textBrowser = new QTextBrowser;
    indexButton = new QPushButton("Index", this);
    backButton = new QPushButton("Back", this);
    forwardButton = new QPushButton("Forward", this);

    QHBoxLayout *toolBarLayout = new QHBoxLayout;
    toolBarLayout->addWidget(indexButton);
    toolBarLayout->addWidget(backButton);
    toolBarLayout->addWidget(forwardButton);
    toolBarLayout->addStretch();

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(toolBarLayout);
    mainLayout->addWidget(textBrowser);
    setLayout(mainLayout);

    connect(indexButton, SIGNAL(clicked()), textBrowser, SLOT(home()));
    connect(backButton, SIGNAL(clicked()), textBrowser, SLOT(backward()));
    connect(forwardButton, SIGNAL(clicked()), textBrowser, SLOT(forward()));
    connect(textBrowser, SIGNAL(backwardAvailable(bool)), backButton, SLOT(setEnabled(bool)));
    connect(textBrowser, SIGNAL(forwardAvailable(bool)), forwardButton, SLOT(setEnabled(bool)));
    connect(textBrowser, SIGNAL(sourceChanged(QUrl)), this, SLOT(updateWindowTitle()));

    textBrowser->setSearchPaths(QStringList() << QDir::currentPath() + "/help");
    textBrowser->setSource(page);
}

void HelpContentsView::setSource(const QString &page)
{
    textBrowser->setSource(page);
}

void HelpContentsView::clearHistory()
{
    textBrowser->clearHistory();
}

void HelpContentsView::updateWindowTitle()
{
    setWindowTitle(tr("Help contents: %1").arg(textBrowser->documentTitle()));
}
