#include "iactioncompleter.h"
#include <abstractobj.h>
#include <QDebug>

IActionCompleter::IActionCompleter(const QAbstractItemModel *sourceModel, QObject *parent) :
    QCompleter(parent)
{
    setCompletionPrefix(".");

    QStandardItemModel *model = new QStandardItemModel(this);
    QStandardItem *rootItem = model->invisibleRootItem();
    for (int i=0; i<sourceModel->rowCount(); i++)
    {
        AbstractObj *obj = static_cast<AbstractObj*>(sourceModel->index(i, 0).internalPointer());
        if (obj->objType() == AbstractObj::Dialog)
        {
            QString dialogName = sourceModel->index(i, 1).data().toString();
            QStandardItem *dialogItem = new QStandardItem;
            dialogItem->setData(dialogName, Qt::UserRole);
            dialogItem->setData(dialogName, Qt::DisplayRole);
            rootItem->appendRow(dialogItem);
            int nObjInDialog = sourceModel->rowCount(sourceModel->index(i, 0));
            QModelIndex dialogParentIndex = sourceModel->index(i, 0);
            for (int j=0; j<nObjInDialog; j++)
            {
                AbstractObj *controlObj
                        = static_cast<AbstractObj*>(sourceModel->index(j, 0,
                          dialogParentIndex).internalPointer());
                AbstractObj::ObjType controlObjType = controlObj->objType();
                switch (controlObjType)
                {
                    case AbstractObj::CheckBox:
                    case AbstractObj::Radio:
                    case AbstractObj::Spinner:
                    case AbstractObj::Edit:
                    case AbstractObj::VarList:
                    case AbstractObj::VarName:
                    case AbstractObj::File:
                    case AbstractObj::ListBox:
                    case AbstractObj::ComboBox:
                    case AbstractObj::Button:
                    case AbstractObj::Text:
                    case AbstractObj::TextBox:
                    case AbstractObj::GroupBox:
                    case AbstractObj::Frame:
                    case AbstractObj::Color:
                    case AbstractObj::Exp:
                        {
                        QString controlName = sourceModel->index(j, 1, dialogParentIndex).data().toString();
                        QString controlTypeString
                                = sourceModel->index(j, 0, dialogParentIndex).data().toString();
                        QStandardItem *controlItem = new QStandardItem;
                        controlItem->setData(controlName + " (" + controlTypeString + ")", Qt::DisplayRole);
                        controlItem->setData(controlName, Qt::UserRole);
                        dialogItem->appendRow(controlItem);
                        controlItem->appendRow(itemForMemberFunction(
                            "hide",
                            "Specifies that the control disappear from view"));
                        controlItem->appendRow(itemForMemberFunction(
                            "show",
                            "Specifies that the control reappear (if it is not already visible)"));
                        controlItem->appendRow(itemForMemberFunction(
                            "disable",
                            "Specifies that the control be disabled (if it is not already)"));
                        controlItem->appendRow(itemForMemberFunction(
                            "enable",
                            "Specifies that the control be enabled (if it is not already"));
                        controlItem->appendRow(itemForMemberFunction(
                            "setposition",
                            "Specifies the new position and size"));
                        switch (controlObjType)
                        {
                            case AbstractObj::CheckBox:
                                //controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(itemForMemberFunction(
                                    "setlabel",
                                    "Sets the label for the edit field"));
                                controlItem->appendRow(new QStandardItem("setoff"));
                                controlItem->appendRow(new QStandardItem("seton"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::Radio:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("seton"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::Spinner:
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("setrange"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::Edit:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("append"));
                                controlItem->appendRow(new QStandardItem("prepend"));
                                controlItem->appendRow(new QStandardItem("insert"));
                                controlItem->appendRow(new QStandardItem("smartinsert"));
                                controlItem->appendRow(new QStandardItem("setfocus"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::VarList:
                            case AbstractObj::VarName:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("append"));
                                controlItem->appendRow(new QStandardItem("prepend"));
                                controlItem->appendRow(new QStandardItem("insert"));
                                controlItem->appendRow(new QStandardItem("smartinsert"));
                                controlItem->appendRow(new QStandardItem("setfocus"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::File:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("append"));
                                controlItem->appendRow(new QStandardItem("prepend"));
                                controlItem->appendRow(new QStandardItem("insert"));
                                controlItem->appendRow(new QStandardItem("smartinsert"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::ListBox:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("setfocus"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("repopulate"));
                                controlItem->appendRow(new QStandardItem("forceselchange"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::ComboBox:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("setfocus"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("repopulate"));
                                controlItem->appendRow(new QStandardItem("forceselchange"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                controlItem->appendRow(new QStandardItem("append"));
                                controlItem->appendRow(new QStandardItem("prepend"));
                                controlItem->appendRow(new QStandardItem("insert"));
                                controlItem->appendRow(new QStandardItem("smartinsert"));
                                break;
                            case AbstractObj::Button:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setfocus"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::Text:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                break;
                            case AbstractObj::TextBox:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                break;
                            case AbstractObj::GroupBox:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                break;
                            case AbstractObj::Frame:
                                // No special member functions provided
                                break;
                            case AbstractObj::Color:
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            case AbstractObj::Exp:
                                controlItem->appendRow(new QStandardItem("setlabel"));
                                controlItem->appendRow(new QStandardItem("setvalue"));
                                controlItem->appendRow(new QStandardItem("append"));
                                controlItem->appendRow(new QStandardItem("prepend"));
                                controlItem->appendRow(new QStandardItem("insert"));
                                controlItem->appendRow(new QStandardItem("smartinsert"));
                                controlItem->appendRow(new QStandardItem("setoption"));
                                controlItem->appendRow(new QStandardItem("setdefault"));
                                controlItem->appendRow(new QStandardItem("settooltip"));
                                break;
                            default:
                                break;
                        }
                        for (int k=0; k<controlItem->rowCount(); k++)
                        {
                            QStandardItem *childOfControlItem = controlItem->child(k);
                            QVariant childOfControlItemDisplayRoleData
                                    = childOfControlItem->data(Qt::DisplayRole);
                            childOfControlItem->setData(childOfControlItemDisplayRoleData, Qt::UserRole);
                        }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    }

    setCompletionRole(Qt::UserRole);
    setModel(model);
}

QString IActionCompleter::pathFromIndex(const QModelIndex &index) const
{
    QString path;
    QModelIndex ind = index;
    path = ind.data(Qt::UserRole).toString();
    ind = ind.parent();
    while (ind != QModelIndex())
    {
        path = ind.data(Qt::UserRole).toString() + "." + path;
        ind = ind.parent();
    }
    return path;
}

QStringList IActionCompleter::splitPath(const QString &path) const
{
    return path.split(".");
}

QStandardItem *IActionCompleter::itemForMemberFunction(const QString &displayData, const QString &toolTip)
{
    QStandardItem *item = new QStandardItem;
    item->setData(displayData, Qt::UserRole);
    item->setData(displayData, Qt::DisplayRole);
    item->setData(toolTip, Qt::ToolTipRole);
    return item;
}
