#include "listcontentmodel.h"

ListContentModel::ListContentModel(QObject *parent, bool isNewNameAutomatic, QSize sizeHint) :
    QAbstractListModel(parent)
{
    this->isNewNameAutomatic = isNewNameAutomatic;
    this->sizeHint = sizeHint;
}

void ListContentModel::setStringList(const QStringList &stringList)
{
    m_stringList = stringList;
}

int ListContentModel::rowCount(const QModelIndex &parent) const
{
    return m_stringList.count();
}

QVariant ListContentModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (index.row() >= m_stringList.size())
        return QVariant();

    switch (role)
    {
        case Qt::DisplayRole:
        case Qt::EditRole:
        case Qt::ToolTipRole:
            return m_stringList.at(index.row());
            break;
        case Qt::SizeHintRole:
            if (sizeHint != QSize())
                return sizeHint;
            else
                return QVariant();
            break;
        default:
            return QVariant();
            break;
    }
}

Qt::ItemFlags ListContentModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return Qt::ItemIsEnabled;

    return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
}

bool ListContentModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid() && role == Qt::EditRole)
    {
        m_stringList.replace(index.row(), value.toString());
        emit dataChanged(index, index);
        return true;
    }
    return false;
}

bool ListContentModel::insertRow(int row, const QModelIndex &parent)
{
    beginInsertRows(parent, row, row);
    if (isNewNameAutomatic)
        m_stringList.insert(row, nameForNewItem());
    else
        m_stringList.insert(row, "");
    endInsertRows();
    return true;
}

bool ListContentModel::removeRow(int row, const QModelIndex &parent)
{
    beginRemoveRows(parent, row, row);
    m_stringList.removeAt(row);
    endRemoveRows();
    return true;
}

void ListContentModel::moveItemUp(int row)
{
    beginMoveRows(QModelIndex(), row, row, QModelIndex(), row - 1);
    m_stringList.swap(row, row - 1);
    endMoveRows();
}

void ListContentModel::moveItemDown(int row)
{
    beginMoveRows(QModelIndex(), row, row, QModelIndex(), row + 2);
    m_stringList.swap(row, row + 1);
    endMoveRows();
}

QStringList ListContentModel::stringList()
{
    return m_stringList;
}

QString ListContentModel::nameForNewItem()
{
    QString prefix = "item";
    int itemCount = m_stringList.count();
    QString objName;
    QList<int> objIdList;
    for (int i=0; i<itemCount; i++)
    {
        objName = index(i, 0).data().toString();
        if (objName.startsWith(prefix))
        {
            bool conversionToIntIsOk = false;
            int number = objName.mid(prefix.length(), -1).toInt(&conversionToIntIsOk);
            if (conversionToIntIsOk)
                objIdList.append(number);
        }
    }

    int freeObjId = 1;
    while (objIdList.contains(freeObjId))
    {
        freeObjId++;
    }

    return QString("%1%2").arg(prefix).arg(freeObjId);
}

















