#include "listcontentproperty.h"
#include "listcontentdelegate.h"
#include <QDebug>

ListContentProperty::ListContentProperty()
{
    setValue("");
}

QVariant ListContentProperty::updateDisplayRoleDataString(const QString &valueString)
{
    return valueString;
}

QString ListContentProperty::updateCommandString(const QString &valueString)
{
    QString items = valueString;
    QStringList itemList = items.split("; ", QString::SkipEmptyParts);
    QString returnString;
    returnString += "\nBEGIN";
    for (int i=0; i<itemList.count(); i++)
        returnString += "\n  " + itemList.at(i);
    returnString += "\nEND";
    return returnString;
}

QString ListContentProperty::domElementName() const
{
    return "content";
}

QVariant ListContentProperty::propertyNameForDisplayRoleData() const
{
    return "Content";
}

bool ListContentProperty::isRequired() const
{
    return true;
}

AbstractDelegate::DelegateType ListContentProperty::delegateForProperty() const
{
    return AbstractDelegate::ListContent;
}

void ListContentProperty::setFromObjDomElement(const QDomElement &domElement)
{
    QDomNode domNode = domElement.namedItem(domElementName());
    if (domNode.isNull())
        setValue("");
    else
    {
        QString string;
        bool atLeastOneItem = false;
        QDomNode itemNode = domNode.toElement().firstChild();
        while (!itemNode.isNull())
        {
            if (itemNode.toElement().tagName() == "listitem")
            {
                if (atLeastOneItem)
                    string += "; " + itemNode.childNodes().at(0).toText().data();
                else
                    string += itemNode.childNodes().at(0).toText().data();
                atLeastOneItem = true;
            }
            itemNode = itemNode.nextSibling();
        }
        setValue(string);
    }
}

QDomElement ListContentProperty::domElement(QDomDocument &xmlDocument) const
{
    QDomElement domElement = xmlDocument.createElement(domElementName());

    if (value().isEmpty())
        return domElement;
    else
    {
        QStringList items = value().split("; ");
        for (int i=0; i<items.count(); i++)
        {
            QDomElement listItemElement = xmlDocument.createElement("listitem");
            QDomText listItemText = xmlDocument.createTextNode(items.at(i));
            listItemElement.appendChild(listItemText);
            domElement.appendChild(listItemElement);
        }
        return domElement;
    }
}

