#include "listcontentsetter.h"
#include <QVBoxLayout>
#include <QHBoxLayout>

ListContentSetter::ListContentSetter(QWidget *parent)
    : AbstractSetter(parent)
{
    // List view
    listView = new ListContentListView;
    connect(listView, SIGNAL(removeItem()), this, SLOT(onRemoveButtonClicked()));

    // Add button
    addButton = new QToolButton;
    addButton->setIcon(QIcon(":/images/icons/add.png"));
    connect(addButton, SIGNAL(clicked()), this, SLOT(onAddButtonClicked()));

    // Remove button
    removeButton = new QToolButton;
    removeButton->setIcon(QIcon(":/images/icons/remove.png"));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(onRemoveButtonClicked()));

    // Up button
    upButton = new QToolButton;
    upButton->setIcon(QIcon(":/images/icons/up.png"));
    connect(upButton, SIGNAL(clicked()), this, SLOT(onUpButtonClicked()));

    // Down button
    downButton = new QToolButton;
    downButton->setIcon(QIcon(":/images/icons/down.png"));
    connect(downButton, SIGNAL(clicked()), this, SLOT(onDownButtonClicked()));

    // Buttons layout
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    buttonsLayout->addStretch();
    buttonsLayout->addWidget(addButton);
    buttonsLayout->addWidget(removeButton);
    buttonsLayout->addWidget(upButton);
    buttonsLayout->addWidget(downButton);

    // Main layout
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);
    mainLayout->addWidget(listView);
    mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);

    // Model
    model = new ListContentModel(this);
    listView->setModel(model);
}

void ListContentSetter::setItemList(const QStringList &stringList)
{
    model->setStringList(stringList);
}

void ListContentSetter::onAddButtonClicked()
{
    int currentRow = listView->currentIndex().row();
    int row;
    if (currentRow == -1)
        row = model->rowCount();
    else
        row = currentRow + 1;
    model->insertRow(row);
    QModelIndex newIndex = model->index(row);
    listView->selectionModel()->clearSelection();
    listView->selectionModel()->setCurrentIndex(newIndex, QItemSelectionModel::Select);
    listView->scrollTo(newIndex);
}

void ListContentSetter::onRemoveButtonClicked()
{
    int currentRow = listView->currentIndex().row();
    if (currentRow != -1)
        model->removeRow(currentRow);
}

void ListContentSetter::onUpButtonClicked()
{
    QModelIndex selectedIndex = listView->currentIndex();
    int rowSelected = selectedIndex.row();
    if (rowSelected != -1 && rowSelected != 0)
    {
        model->moveItemUp(rowSelected);
        QModelIndex newSelectedIndex = model->index(rowSelected - 1);
        listView->selectionModel()->setCurrentIndex(newSelectedIndex, QItemSelectionModel::Select);
        listView->scrollTo(newSelectedIndex);
    }
}

void ListContentSetter::onDownButtonClicked()
{
    QModelIndex selectedIndex = listView->currentIndex();
    int rowSelected = selectedIndex.row();
    if (rowSelected != -1 && rowSelected < model->rowCount() - 1)
    {
        model->moveItemDown(rowSelected);
        QModelIndex newSelectedIndex = model->index(rowSelected + 1);
        listView->selectionModel()->setCurrentIndex(newSelectedIndex, QItemSelectionModel::Select);
        listView->scrollTo(newSelectedIndex);
    }
}

QStringList ListContentSetter::itemList()
{
    return model->stringList();
}


