#ifndef MODEL_H
#define MODEL_H

#include <QAbstractItemModel>
#include <QString>
#include "document.h"

// Objects
#include "versionobj.h"
#include "includeobj.h"
#include "defineobj.h"
#include "positionobj.h"
#include "listobj.h"
#include "dialogobj.h"
#include "checkboxobj.h"
#include "radioobj.h"
#include "spinnerobj.h"
#include "editobj.h"
#include "varlistobj.h"
#include "varnameobj.h"
#include "fileobj.h"
#include "listboxobj.h"
#include "comboboxobj.h"
#include "buttonobj.h"
#include "textobj.h"
#include "textboxobj.h"
#include "groupboxobj.h"
#include "frameobj.h"
#include "colorobj.h"
#include "expobj.h"
#include "okobj.h"
#include "submitobj.h"
#include "cancelobj.h"
#include "copyobj.h"
#include "helpobj.h"
#include "resetobj.h"
#include "programobj.h"
#include "scriptobj.h"

class Model : public QAbstractItemModel
{
Q_OBJECT

public:
    explicit Model(QObject *parent = 0);

    // Virtual functions
    virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &index) const;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
    virtual bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());
    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
    virtual QStringList mimeTypes() const;

    // Other functions
    void setDocument(Document *document);
    bool loadVisuaFile(const QString &fileName);
    void clear();
    void insertNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int row);
    void insertNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int row,
                      int x, int y);
    void appendNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex);
    void appendNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int x, int y);
    void insertObjList(QList<AbstractObj*> objList, int row, const QModelIndex &parentIndex);
    void appendObjList(QList<AbstractObj*> objList, const QModelIndex &parentIndex);

private:
    // Document
    Document *m_document;

    // Functions
    AbstractObj *newObjectOfType(AbstractObj::ObjType objType);
    QString prefixNameForNewObj(AbstractObj::ObjType objType);
    QString newNameForObj(const QModelIndex &parent, AbstractObj::ObjType objType);

signals:
    void modelWasResetted();

public slots:
    bool moveItemUp(const QModelIndex &parent, int row);
    bool moveItemDown(const QModelIndex &parent, int row);
};

#endif // MODEL_H
