#include "objectboxmodel.h"
#include <QStringList>
#include "objtypemimedata.h"
#include <QStringListModel>
#include <QDebug>

ObjectBoxModel::ObjectBoxModel(QObject *parent) :
    QAbstractItemModel(parent)
{
    setSupportedDragActions(Qt::CopyAction);

    m_rootItem = new ObjectBoxItem;

    QVariant variantData;
    variantData.setValue(AbstractObj::CheckBox);

    // Controls
    QStringList controlsNameList;
    controlsNameList << "Checkbox" << "Radio" << "Spinner" << "Edit" << "Varlist" << "Varname"
                     << "File" << "Listbox" << "Combobox" << "Button" << "Text" << "Textbox"
                     << "Groupbox" << "Frame" << "Color" << "Exp";
    QString controlsFilterString;
    for (int i=0; i<controlsNameList.count(); i++)
        controlsFilterString += controlsNameList.at(i) +  " ";
    ObjectBoxItem *controlsBoxItem = new ObjectBoxItem("Controls", controlsFilterString);
    variantData.setValue(AbstractObj::CheckBox);
    controlsBoxItem->appendItem(new ObjectBoxItem("Checkbox", "Checkbox", variantData));
    variantData.setValue(AbstractObj::Radio);
    controlsBoxItem->appendItem(new ObjectBoxItem("Radio", "Radio", variantData));
    variantData.setValue(AbstractObj::Spinner);
    controlsBoxItem->appendItem(new ObjectBoxItem("Spinner", "Spinner", variantData));
    variantData.setValue(AbstractObj::Edit);
    controlsBoxItem->appendItem(new ObjectBoxItem("Edit", "Edit", variantData));
    variantData.setValue(AbstractObj::VarList);
    controlsBoxItem->appendItem(new ObjectBoxItem("Varlist", "Varlist", variantData));
    variantData.setValue(AbstractObj::VarName);
    controlsBoxItem->appendItem(new ObjectBoxItem("Varname", "Varname", variantData));
    variantData.setValue(AbstractObj::File);
    controlsBoxItem->appendItem(new ObjectBoxItem("File", "File", variantData));
    variantData.setValue(AbstractObj::ListBox);
    controlsBoxItem->appendItem(new ObjectBoxItem("Listbox", "Listbox", variantData));
    variantData.setValue(AbstractObj::ComboBox);
    controlsBoxItem->appendItem(new ObjectBoxItem("Combobox", "Combobox", variantData));
    variantData.setValue(AbstractObj::Button);
    controlsBoxItem->appendItem(new ObjectBoxItem("Button", "Button", variantData));
    variantData.setValue(AbstractObj::Text);
    controlsBoxItem->appendItem(new ObjectBoxItem("Text", "Text", variantData));
    variantData.setValue(AbstractObj::TextBox);
    controlsBoxItem->appendItem(new ObjectBoxItem("Textbox", "Textbox", variantData));
    variantData.setValue(AbstractObj::GroupBox);
    controlsBoxItem->appendItem(new ObjectBoxItem("Groupbox", "Groupbox", variantData));
    variantData.setValue(AbstractObj::Frame);
    controlsBoxItem->appendItem(new ObjectBoxItem("Frame", "Frame", variantData));
    variantData.setValue(AbstractObj::Color);
    controlsBoxItem->appendItem(new ObjectBoxItem("Color", "Color", variantData));
    variantData.setValue(AbstractObj::Exp);
    controlsBoxItem->appendItem(new ObjectBoxItem("Exp", "Exp", variantData));
    m_rootItem->appendItem(controlsBoxItem);

    // Special buttons
    QStringList specialButtonsNameList;
    specialButtonsNameList << "Ok" << "Submit" << "Cancel" << "Copy" << "Help" << "Reset";
    QString specialButtonsFilterString;
    for (int i=0; i<specialButtonsNameList.count(); i++)
        specialButtonsFilterString += specialButtonsNameList.at(i) +  " ";
    ObjectBoxItem *specialButtonsBoxItem = new ObjectBoxItem("Special buttons", specialButtonsFilterString);
    variantData.setValue(AbstractObj::Ok);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Ok", "Ok", variantData));
    variantData.setValue(AbstractObj::Submit);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Submit", "Submit", variantData));
    variantData.setValue(AbstractObj::Cancel);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Cancel", "Cancel", variantData));
    variantData.setValue(AbstractObj::Copy);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Copy", "Copy", variantData));
    variantData.setValue(AbstractObj::Help);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Help", "Help", variantData));
    variantData.setValue(AbstractObj::Reset);
    specialButtonsBoxItem->appendItem(new ObjectBoxItem("Reset", "Reset", variantData));
    m_rootItem->appendItem(specialButtonsBoxItem);

    // Miscellaneous
    QStringList miscellaneousNameList;
    miscellaneousNameList << "Version" << "Include" << "Define" << "Position" << "List" << "Dialog"
                          << "Script";
    QString miscellaneousFilterString;
    for (int i=0; i<miscellaneousNameList.count(); i++)
        miscellaneousFilterString += miscellaneousNameList.at(i) +  " ";
    ObjectBoxItem *miscellaneousBoxItem = new ObjectBoxItem("Miscellaneous", miscellaneousFilterString);
    variantData.setValue(AbstractObj::Version);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Version", "Version", variantData));
    variantData.setValue(AbstractObj::Include);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Include", "Include", variantData));
    variantData.setValue(AbstractObj::Define);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Define", "Define", variantData));
    variantData.setValue(AbstractObj::Position);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Position", "Position", variantData));
    variantData.setValue(AbstractObj::List);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("List", "List", variantData));
    variantData.setValue(AbstractObj::Dialog);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Dialog", "Dialog", variantData));
    variantData.setValue(AbstractObj::Script);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Script", "Script", variantData));
    variantData.setValue(AbstractObj::Program);
    miscellaneousBoxItem->appendItem(new ObjectBoxItem("Program", "Program", variantData));
    m_rootItem->appendItem(miscellaneousBoxItem);
}

QModelIndex ObjectBoxModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    if (!parent.isValid())
    {
        ObjectBoxItem *item = m_rootItem->itemAt(row);
        return createIndex(row, column, item);
    }
    else
    {
        ObjectBoxItem *parentItem = static_cast<ObjectBoxItem*>(parent.internalPointer());
        ObjectBoxItem *childItem = parentItem->itemAt(row);
        return createIndex(row, column, childItem);
    }
}

QModelIndex ObjectBoxModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    ObjectBoxItem *childItem = static_cast<ObjectBoxItem*>(index.internalPointer());
    ObjectBoxItem *parentItem = childItem->parent();
    if (parentItem == m_rootItem)
        return QModelIndex();
    else
        return createIndex(parentItem->row(), 0, parentItem);
}

int ObjectBoxModel::rowCount(const QModelIndex &parent) const
{
    // Only indexes at column 0 have child indexes
    if (parent.column() > 0)
        return 0;

    if (!parent.isValid())
        return m_rootItem->childCount();
    else
    {
        ObjectBoxItem *item = static_cast<ObjectBoxItem*>(parent.internalPointer());
        return item->childCount();
    }
}

int ObjectBoxModel::columnCount(const QModelIndex &parent) const
{
    return 1;
}

QVariant ObjectBoxModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    ObjectBoxItem *item = static_cast<ObjectBoxItem*>(index.internalPointer());
    switch (role)
    {
        case Qt::DisplayRole:
            return item->string();
            break;
        case Qt::EditRole:
            return item->filterString();
            break;
        case Qt::UserRole:
            return item->objType();
            break;
        case Qt::DecorationRole:
            {
            if (index.parent().isValid())
            {
                AbstractObj::ObjType objType = item->objType().value<AbstractObj::ObjType>();
                switch (objType)
                {
                    case AbstractObj::Version:
                        return QIcon(":images/icons/objicons/version.png");
                        break;
                    case AbstractObj::Include:
                        return QIcon(":images/icons/objicons/include.png");
                        break;
                    case AbstractObj::Define:
                        return QIcon(":images/icons/objicons/define.png");
                        break;
                    case AbstractObj::Position:
                        return QIcon(":images/icons/objicons/position.png");
                        break;
                    case AbstractObj::List:
                        return QIcon(":images/icons/objicons/list.png");
                        break;
                    case AbstractObj::Dialog:
                        return QIcon(":images/icons/objicons/dialog.png");
                        break;
                    case AbstractObj::CheckBox:
                        return QIcon(":images/icons/objicons/checkbox.png");
                        break;
                    case AbstractObj::Radio:
                        return QIcon(":images/icons/objicons/radio.png");
                        break;
                    case AbstractObj::Spinner:
                        return QIcon(":images/icons/objicons/spinner.png");
                        break;
                    case AbstractObj::Edit:
                        return QIcon(":images/icons/objicons/edit.png");
                        break;
                    case AbstractObj::VarList:
                        return QIcon(":images/icons/objicons/varlist.png");
                        break;
                    case AbstractObj::VarName:
                        return QIcon(":images/icons/objicons/varname.png");
                        break;
                    case AbstractObj::File:
                        return QIcon(":images/icons/objicons/file.png");
                        break;
                    case AbstractObj::ListBox:
                        return QIcon(":images/icons/objicons/listbox.png");
                        break;
                    case AbstractObj::ComboBox:
                        return QIcon(":images/icons/objicons/combobox.png");
                        break;
                    case AbstractObj::Button:
                        return QIcon(":images/icons/objicons/button.png");
                        break;
                    case AbstractObj::Text:
                        return QIcon(":images/icons/objicons/text.png");
                        break;
                    case AbstractObj::TextBox:
                        return QIcon(":images/icons/objicons/textbox.png");
                        break;
                    case AbstractObj::GroupBox:
                        return QIcon(":images/icons/objicons/groupbox.png");
                        break;
                    case AbstractObj::Frame:
                        return QIcon(":images/icons/objicons/frame.png");
                        break;
                    case AbstractObj::Color:
                        return QIcon(":images/icons/objicons/color.png");
                        break;
                    case AbstractObj::Exp:
                        return QIcon(":images/icons/objicons/exp.png");
                        break;
                    case AbstractObj::Ok:
                        return QIcon(":images/icons/objicons/ok.png");
                        break;
                    case AbstractObj::Submit:
                        return QIcon(":images/icons/objicons/submit.png");
                        break;
                    case AbstractObj::Cancel:
                        return QIcon(":images/icons/objicons/cancel.png");
                        break;
                    case AbstractObj::Copy:
                        return QIcon(":images/icons/objicons/copy.png");
                        break;
                    case AbstractObj::Help:
                        return QIcon(":images/icons/objicons/help.png");
                        break;
                    case AbstractObj::Reset:
                        return QIcon(":images/icons/objicons/reset.png");
                        break;
                    case AbstractObj::Script:
                        return QIcon(":images/icons/objicons/script.png");
                        break;
                    case AbstractObj::Program:
                        return QIcon(":images/icons/objicons/program.png");
                        break;
                    default:
                        return QIcon(":images/icons/test16x16.png");
                        break;
                }
            }
            else
                return QVariant();
            }
            break;
        default:
            return QVariant();
            break;
    }
}

Qt::ItemFlags ObjectBoxModel::flags(const QModelIndex &index) const
{
    Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);

    if (index.isValid())
        return Qt::ItemIsDragEnabled | defaultFlags;
    else
        return defaultFlags;
}

QMimeData *ObjectBoxModel::mimeData(const QModelIndexList &indexes) const
{
    QVariant variantData = data(indexes.at(0), Qt::UserRole);
    AbstractObj::ObjType objType = variantData.value<AbstractObj::ObjType>();
    ObjTypeMimeData* objTypeMimeData = new ObjTypeMimeData(objType);
    return objTypeMimeData;
}
