#include "objectlistview.h"
#include <QMenu>
#include <QDebug>
#include "model.h"
#include "objectlistproxymodel.h"

ObjectListView::ObjectListView(QWidget *parent) :
    QTreeView(parent)
{
    setAcceptDrops(true);
    setDropIndicatorShown(true);

    // Context menu
    contextMenu = new QMenu(this);
    addControlMenu = new QMenu("Add control", this);
    contextMenu->addMenu(addControlMenu);

    // Actions for context menu
    QVariant controlData;
    // Checkbox
    addCheckboxAction = new QAction("Checkbox", this);
    controlData.setValue(AbstractObj::CheckBox);
    addCheckboxAction->setData(controlData);
    addCheckboxAction->setIcon(QIcon(":images/icons/objicons/checkbox.png"));
    connect(addCheckboxAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addCheckboxAction);
    // Radio
    addRadioAction = new QAction("Radio button", this);
    controlData.setValue(AbstractObj::Radio);
    addRadioAction->setData(controlData);
    addRadioAction->setIcon(QIcon(":images/icons/objicons/radio.png"));
    connect(addRadioAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addRadioAction);
    // Spinner
    addSpinnerAction = new QAction("Spinner", this);
    controlData.setValue(AbstractObj::Spinner);
    addSpinnerAction->setData(controlData);
    addSpinnerAction->setIcon(QIcon(":images/icons/objicons/spinner.png"));
    connect(addSpinnerAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addSpinnerAction);
    // Edit
    addEditAction = new QAction("Edit", this);
    controlData.setValue(AbstractObj::Edit);
    addEditAction->setData(controlData);
    addEditAction->setIcon(QIcon(":images/icons/objicons/edit.png"));
    connect(addEditAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addEditAction);
    // Varlist
    addVarlistAction = new QAction("Varlist", this);
    controlData.setValue(AbstractObj::VarList);
    addVarlistAction->setData(controlData);
    addVarlistAction->setIcon(QIcon(":images/icons/objicons/varlist.png"));
    connect(addVarlistAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addVarlistAction);
    // Varname
    addVarnameAction = new QAction("Varname", this);
    controlData.setValue(AbstractObj::VarName);
    addVarnameAction->setData(controlData);
    addVarnameAction->setIcon(QIcon(":images/icons/objicons/varname.png"));
    connect(addVarnameAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addVarnameAction);
    // File
    addFileAction = new QAction("File", this);
    controlData.setValue(AbstractObj::File);
    addFileAction->setData(controlData);
    addFileAction->setIcon(QIcon(":images/icons/objicons/file.png"));
    connect(addFileAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addFileAction);
    // Listbox
    addListboxAction = new QAction("List box", this);
    controlData.setValue(AbstractObj::ListBox);
    addListboxAction->setData(controlData);
    addListboxAction->setIcon(QIcon(":images/icons/objicons/listbox.png"));
    connect(addListboxAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addListboxAction);
    // Combobox
    addComboboxAction = new QAction("Combo box", this);
    controlData.setValue(AbstractObj::ComboBox);
    addComboboxAction->setData(controlData);
    addComboboxAction->setIcon(QIcon(":images/icons/objicons/combobox.png"));
    connect(addComboboxAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addComboboxAction);
    // Button
    addButtonAction = new QAction("Button", this);
    controlData.setValue(AbstractObj::Button);
    addButtonAction->setData(controlData);
    addButtonAction->setIcon(QIcon(":images/icons/objicons/button.png"));
    connect(addButtonAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addButtonAction);
    // Text
    addTextAction = new QAction("Text", this);
    controlData.setValue(AbstractObj::Text);
    addTextAction->setData(controlData);
    addTextAction->setIcon(QIcon(":images/icons/objicons/text.png"));
    connect(addTextAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addTextAction);
    // Textbox
    addTextboxAction = new QAction("Text box", this);
    controlData.setValue(AbstractObj::TextBox);
    addTextboxAction->setData(controlData);
    addTextboxAction->setIcon(QIcon(":images/icons/objicons/textbox.png"));
    connect(addTextboxAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addTextboxAction);
    // Groupbox
    addGroupboxAction = new QAction("Group box", this);
    controlData.setValue(AbstractObj::GroupBox);
    addGroupboxAction->setData(controlData);
    addGroupboxAction->setIcon(QIcon(":images/icons/objicons/groupbox.png"));
    connect(addGroupboxAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addGroupboxAction);
    // Frame
    addFrameAction = new QAction("Frame", this);
    controlData.setValue(AbstractObj::Frame);
    addFrameAction->setData(controlData);
    addFrameAction->setIcon(QIcon(":images/icons/objicons/frame.png"));
    connect(addFrameAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addFrameAction);
    // Color
    addColorAction = new QAction("Color", this);
    controlData.setValue(AbstractObj::Color);
    addColorAction->setData(controlData);
    addColorAction->setIcon(QIcon(":images/icons/objicons/color.png"));
    connect(addColorAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addColorAction);
    // Exp
    addExpAction = new QAction("Expression", this);
    controlData.setValue(AbstractObj::Exp);
    addExpAction->setData(controlData);
    addExpAction->setIcon(QIcon(":images/icons/objicons/exp.png"));
    connect(addExpAction, SIGNAL(triggered()), this, SLOT(onAddControlActionTriggered()));
    addControlMenu->addAction(addExpAction);

    // Delete action
    deleteObjectAction = new QAction("Delete object", this);
    contextMenu->addAction(deleteObjectAction);
    connect(deleteObjectAction, SIGNAL(triggered()), this, SLOT(onDeleteObjectActionTriggered()));
}

void ObjectListView::contextMenuEvent(QContextMenuEvent *event)
{
    if (selectionModel()->selectedIndexes().isEmpty())
    {
        addControlMenu->setDisabled(true);
        deleteObjectAction->setDisabled(true);   
    }
    else
    {
        QModelIndex selectedIndex = selectionModel()->selectedIndexes().at(0);
        AbstractObj* selectedObj = static_cast<AbstractObj*>(selectedIndex.internalPointer());
        if (selectedObj->objType() != AbstractObj::Dialog)
            addControlMenu->setDisabled(true);
        else
            addControlMenu->setDisabled(false);
        deleteObjectAction->setDisabled(false);
    }
    contextMenu->exec(event->globalPos());
}

void ObjectListView::onDeleteObjectActionTriggered()
{
    QModelIndex selectedIndex = selectedIndexes().at(0);
    QModelIndex parentIndex = selectedIndex.parent();
    model()->removeRows(selectedIndex.row(), 1, parentIndex);
}

void ObjectListView::onAddControlActionTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    AbstractObj::ObjType objType = action->data().value<AbstractObj::ObjType>();
    Model *innerModel = static_cast<Model*>(model());
    innerModel->appendNewObj(objType, selectedIndexes().at(0));
    emit controlAddedToDialog();
}

void ObjectListView::onSelectionChangedInCanvas(const QItemSelection &selected,
                                                const QItemSelection &deselected)
{
    selectionModel()->clear();

    if (selected.count() == 0)
    {
        if (!deselected.isEmpty())
        {
            if (!deselected.at(0).indexes().isEmpty())
            {
                ObjectListProxyModel *proxyModel = static_cast<ObjectListProxyModel*>(model());
                QModelIndex deselectedIndex = proxyModel->mapFromSource(deselected.at(0).indexes().at(0));
                QModelIndex dialogIndex = deselectedIndex.parent();
                setCurrentIndex(dialogIndex);
            }
        }
    }
    else
    {
        ObjectListProxyModel *proxyModel = static_cast<ObjectListProxyModel*>(model());
        QModelIndex indexToBeSelected = proxyModel->mapFromSource(selected.at(0).indexes().at(0));
        setCurrentIndex(indexToBeSelected);
    }
}

void ObjectListView::onUpButtonClicked()
{
    QModelIndex currentParentIndex = currentIndex().parent();
    int currentRow = currentIndex().row();

    // Check if no item is selected
    if (currentRow == -1) return;

    // Check if first item is selected
    if (currentRow == 0) return;

    ObjectListProxyModel *proxyModel = static_cast<ObjectListProxyModel*>(model());
    proxyModel->moveItemUp(currentParentIndex, currentRow);
    emit itemMovedUp();
}

void ObjectListView::onDownButtonClicked()
{
    QModelIndex currentParentIndex = currentIndex().parent();
    int currentRow = currentIndex().row();

    // Check if no item is selected
    if (currentRow == -1) return;

    // Check if last item is selected
    int rowsOfCurrentParentIndex = model()->rowCount(currentParentIndex);
    if (currentRow == rowsOfCurrentParentIndex - 1) return;

    ObjectListProxyModel *proxyModel = static_cast<ObjectListProxyModel*>(model());
    proxyModel->moveItemDown(currentParentIndex, currentRow);
    emit itemMovedDown();
}
