#include "positiongeometrydatasetter.h"
#include <QHBoxLayout>

PositionGeometryDataSetter::PositionGeometryDataSetter(bool defaultValueAllowed, QWidget *parent)
    : AbstractSetter(parent)
{
    m_defaultValueAllowed = defaultValueAllowed;

    // Horizontal layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Spin box
    spinBox = new QSpinBox(this);
    spinBox->setMinimum(0);
    spinBox->setMaximum(9999);
    layout->addWidget(spinBox);

    // Combo box
    if (defaultValueAllowed)
    {
        comboBox = new QComboBox(this);
        comboBox->addItem("User defined value");
        comboBox->addItem("Default value");
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentComboBoxItemChanged(int)));
        layout->addWidget(comboBox);
    }
}

void PositionGeometryDataSetter::onCurrentComboBoxItemChanged(int pos)
{
    switch (pos)
    {
        case 0: // User defined value
            spinBox->show();
            break;
        case 1: // Default value
            spinBox->hide();
            break;
        default:
            break;
    }
}

void PositionGeometryDataSetter::setPositionGeometryData(const QString &data)
{
    bool isNumeric;
    int numericValue = data.toInt(&isNumeric);

    if (isNumeric && data.at(0) != '+' && data.at(0) != '-') // User defined value
    {
        spinBox->setValue(numericValue);
        spinBox->show();
    }
    else if (data == ".") // Default value
    {
        spinBox->hide();
        comboBox->setCurrentIndex(1);
    }
}

QString PositionGeometryDataSetter::positionGeometryData() const
{
    if (!m_defaultValueAllowed)
        return QString("%1").arg(spinBox->value());

    switch (comboBox->currentIndex())
    {
        case 0: // User defined value
            return QString("%1").arg(spinBox->value());
            break;
        case 1: // Default value
            return ".";
            break;
        default:
            return QString();
            break;
    }
}
