#include "scriptiactionlistsetter.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include "iactionforscriptitemdelegate.h"

ScriptIActionListSetter::ScriptIActionListSetter(const QAbstractItemModel *sourceModel, QWidget *parent)
    : AbstractSetter(parent)
{
    // Source model
    m_sourceModel = sourceModel;

    // Main layout
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);
    //mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);

    // List view
    listView = new QListView;
    mainLayout->addWidget(listView);

    // Add button
    addButton = new QToolButton;
    addButton->setIcon(QIcon(":/images/icons/add.png"));
    connect(addButton, SIGNAL(clicked()), this, SLOT(onAddButtonClicked()));

    // Remove button
    removeButton = new QToolButton;
    removeButton->setIcon(QIcon(":/images/icons/remove.png"));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(onRemoveButtonClicked()));

    // Up button
    upButton = new QToolButton;
    upButton->setIcon(QIcon(":/images/icons/up.png"));
    connect(upButton, SIGNAL(clicked()), this, SLOT(onUpButtonClicked()));

    // Down button
    downButton = new QToolButton;
    downButton->setIcon(QIcon(":/images/icons/down.png"));
    connect(downButton, SIGNAL(clicked()), this, SLOT(onDownButtonClicked()));

    // Buttons layout
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    buttonsLayout->addStretch();
    buttonsLayout->addWidget(addButton);
    buttonsLayout->addWidget(removeButton);
    buttonsLayout->addWidget(upButton);
    buttonsLayout->addWidget(downButton);
    mainLayout->addLayout(buttonsLayout);
}

void ScriptIActionListSetter::setIActionList(const QStringList &list)
{
    m_model = new ListContentModel(this, false, QSize(10, 20));
    m_model->setStringList(list);
    listView->setModel(m_model);

    IActionForScriptItemDelegate *iActionForScriptItemDelegate
            = new IActionForScriptItemDelegate(m_sourceModel);
    listView->setItemDelegate(iActionForScriptItemDelegate);
}

QStringList ScriptIActionListSetter::iActionList()
{
    return m_model->stringList();
}

void ScriptIActionListSetter::onAddButtonClicked()
{
    int currentRow = listView->currentIndex().row();
    int row;
    if (currentRow == -1)
        row = m_model->rowCount();
    else
        row = currentRow + 1;
    m_model->insertRow(row);
    QModelIndex newIndex = m_model->index(row);
    listView->selectionModel()->clearSelection();
    listView->selectionModel()->setCurrentIndex(newIndex, QItemSelectionModel::Select);
    listView->scrollTo(newIndex);
}

void ScriptIActionListSetter::onRemoveButtonClicked()
{
    int currentRow = listView->currentIndex().row();
    if (currentRow != -1)
        m_model->removeRow(currentRow);
}

void ScriptIActionListSetter::onUpButtonClicked()
{
    QModelIndex selectedIndex = listView->currentIndex();
    int rowSelected = selectedIndex.row();
    if (rowSelected != -1 && rowSelected != 0)
    {
        m_model->moveItemUp(rowSelected);
        QModelIndex newSelectedIndex = m_model->index(rowSelected - 1);
        listView->selectionModel()->setCurrentIndex(newSelectedIndex, QItemSelectionModel::Select);
        listView->scrollTo(newSelectedIndex);
    }
}

void ScriptIActionListSetter::onDownButtonClicked()
{
    QModelIndex selectedIndex = listView->currentIndex();
    int rowSelected = selectedIndex.row();
    if (rowSelected != -1 && rowSelected < m_model->rowCount() - 1)
    {
        m_model->moveItemDown(rowSelected);
        QModelIndex newSelectedIndex = m_model->index(rowSelected + 1);
        listView->selectionModel()->setCurrentIndex(newSelectedIndex, QItemSelectionModel::Select);
        listView->scrollTo(newSelectedIndex);
    }
}
