#include "statapathdialog.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QFileDialog>

StataPathDialog::StataPathDialog(const QString &stataPath, QWidget *parent) :
    QDialog(parent)
{
    setWindowTitle("Visua");
    resize(400, height());

    // Layout
    QVBoxLayout *layout = new QVBoxLayout(this);
    setLayout(layout);

    // Up horizontal Layout
    QHBoxLayout *upHorizontalLayout = new QHBoxLayout;
    layout->addLayout(upHorizontalLayout);

    // Bottom horizontal Layout
    QHBoxLayout *bottomHorizontalLayout = new QHBoxLayout;
    layout->addLayout(bottomHorizontalLayout);

    // Label
    QLabel *label = new QLabel("Stata path:", this);
    upHorizontalLayout->addWidget(label);

    // Line edit
    lineEdit = new QLineEdit(stataPath, this);
    lineEdit->setText(stataPath);
    lineEdit->setReadOnly(true);
    upHorizontalLayout->addWidget(lineEdit);

    // Stretch
    bottomHorizontalLayout->addStretch();

    // Cancel button
    QPushButton *cancelButton = new QPushButton("Cancel", this);
    bottomHorizontalLayout->addWidget(cancelButton);
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    // Find button
    QPushButton *findButton = new QPushButton("Find...", this);
    bottomHorizontalLayout->addWidget(findButton);
    connect(findButton, SIGNAL(clicked()), this, SLOT(onFindButtonClicked()));

    // Ok button
    QPushButton *okButton = new QPushButton("Ok", this);
    bottomHorizontalLayout->addWidget(okButton);
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
}

void StataPathDialog::onFindButtonClicked()
{
    QString fileName = QFileDialog::getOpenFileName(
                this,
                "Find Stata executable",
                ".",
                "Executable program (*.exe)");

    if (!fileName.isEmpty())
        lineEdit->setText(fileName);
}

QString StataPathDialog::getStataPath()
{
    return lineEdit->text().trimmed();
}
