#include "uactiondelegate.h"
#include "uactionsetter.h"
#include "abstractobj.h"
#include <QSortFilterProxyModel>

UActionDelegate::UActionDelegate(QObject *parent)
    : AbstractDelegate(parent)
{
}

QWidget *UActionDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                      const QModelIndex &index) const
{
    Q_UNUSED(option)

    // Retrive source model
    const QSortFilterProxyModel *proxyModel = static_cast<const QSortFilterProxyModel*>(index.model());
    QModelIndex sourceIndex = proxyModel->mapToSource(index);
    const QAbstractItemModel *sourceModel = sourceIndex.model();

    // Program list
    QStringList programList;
    for (int i=0; i<sourceModel->rowCount(); i++)
    {
        QModelIndex index = sourceModel->index(i, 0);
        AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
        if (obj->objType() == AbstractObj::Program)
        {
            QModelIndex indexForName = sourceModel->index(i, 1);
            programList << indexForName.data(Qt::EditRole).toString();
        }
    }

    // UAction setter
    UActionSetter *uActionSetter = new UActionSetter(parent, programList);
    return uActionSetter;
}

void UActionDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    UActionSetter *uActionSetter = static_cast<UActionSetter*>(editor);
    uActionSetter->setUActionData(index.data(Qt::EditRole).toString());
}

void UActionDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    UActionSetter *uActionSetter = static_cast<UActionSetter*>(editor);
    model->setData(index, uActionSetter->uActionData());
}

