#include "valuesdelegate.h"

#include "valuessetter.h"
#include <QSortFilterProxyModel>
#include "abstractobj.h"
#include "listobj.h"

ValuesDelegate::ValuesDelegate(QObject *parent)
    : AbstractDelegate(parent)
{
}

QWidget *ValuesDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,
              const QModelIndex &index) const
{
    Q_UNUSED(option)
    const QSortFilterProxyModel *proxyModel = static_cast<const QSortFilterProxyModel*>(index.model());
    QModelIndex sourceIndex = proxyModel->mapToSource(index);
    const QAbstractItemModel *sourceModel = sourceIndex.model();
    QStringList userDefinedLists;
    for (int i=0; i<sourceModel->rowCount(); i++)
    {
        QModelIndex ind = sourceModel->index(i, 0, QModelIndex());
        AbstractObj *obj = static_cast<AbstractObj*>(ind.internalPointer());
        if (obj->objType() == AbstractObj::List)
        {
            ListObj *listObj = static_cast<ListObj*>(obj);
            userDefinedLists << listObj->nameProperty->editRoleData().toString();
        }
    }
    ValuesSetter *valuesSetter = new ValuesSetter(userDefinedLists, parent);
    return valuesSetter;
}

void ValuesDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    ValuesSetter *valuesSetter = static_cast<ValuesSetter*>(editor);
    valuesSetter->setValuesData(index.data(Qt::EditRole).toString());
}

void ValuesDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                            const QModelIndex &index) const
{
    ValuesSetter *valuesSetter = static_cast<ValuesSetter*>(editor);
    model->setData(index, valuesSetter->valuesData());
}
